set more off

local s `1'
local other1 `2'
local other2 `3'

set matsize 500

*Estimate the variance of m (see appendix for the definition of m)
su m2
global sigmasqm=r(sd)^2

*estimate of tau*sigmasqu (defined in appendix as well)
gen ones=1
local todrop `todrop' ones

egen n1=sum(ones), by(sid1)
replace n1=n1-1
egen n2=sum(ones), by(sid2)
replace n2=n2-1
egen n3=sum(ones), by(sid1 sid2)
replace n3=n3-1
replace n1=n1-n3
replace n2=n2-n3

local todrop `todrop' n*

egen partialsum1=sum(m2), by(sid1)
replace partialsum1=partialsum1-m2
egen partialsum2=sum(m2), by(sid2)
replace partialsum2=partialsum2-m2
egen partialsum3=sum(m2), by(sid1 sid2)
replace partialsum3=partialsum3-m2
replace partialsum1=(partialsum1-partialsum3)/n1 if n1!=0
replace partialsum1=0 if n1==0
replace partialsum2=(partialsum2-partialsum3)/n2 if n2!=0
replace partialsum2=0 if n2==0
replace partialsum3=partialsum3/2/n3 if n3!=0
replace partialsum3=0 if n3==0
gen partialsum=partialsum1+partialsum2+partialsum3
gen product=m2*partialsum

local todrop `todrop' partialsum* product

su product
global tausigmasqu=r(mean)

*From appendix, see that the covariance in the error term across observations depends on how many time periods two observations were in the same school for.
*Here Prepare data so that can build the covariance matrix in mata below
*There is probably a much better way to do this, but I think this works.


*Sort data so that if you switched schools (mobile=1) you are at the bottom, and within "mobile=0" and "mobile=1", observations are sorted by first school attended and second school attended

gen mobile=(sid2!=sid1)

sort mobile sid1 sid2 childcode

local todrop `todrop' mobile

su sid1
local endschool=r(max)
local startschool=r(min)
su sid2
local endschool=max(`endschool',r(max))
local startschool=min(`startschool',r(min))
local numschool=`endschool'-`startschool'+1
matrix sizeschool1=J(500,1,0)
local extranumschool=`numschool'-500
matrix sizeschool2=J(`extranumschool',1,0)

*Count how many observations have not switched school over the two time periods for all schools

local count=1
forvalues i=`startschool'/`endschool' {
	su ones if sid1==sid2&sid2==`i'
	if r(N)==0 {
		di "ZEROSIZE"
	}
	if `count'<=500 {
	matrix sizeschool1[`count',1]=r(N)
	}
	else {
	local temp=`count'-500
	matrix sizeschool2[`temp',1]=r(N)
	}
	local count=`count'+1
}

*For the observations that switched schools, keep track of which school was attended in first and second time period.
su ones if mobile
local nummobile=r(N)

gen idmobile=0
local todrop `todrop' idmobile

replace idmobile=1 if mobile
replace idmobile=idmobile[_n-1]+1 if mobile[_n-1]==1

matrix schoolmobile1=J(`nummobile',1,0)
matrix schoolmobile2=J(`nummobile',1,0)

forvalues i=1/`nummobile' {
	su sid1 if idmobile==`i'
	matrix schoolmobile1[`i',1]=r(mean)
	su sid2 if idmobile==`i'
	matrix schoolmobile2[`i',1]=r(mean)
}

*List control variables

forvalues t=0/2 {
	global controls`t'
	foreach var in $controls {
		global controls`t' ${controls`t'} `var'`t'
	}
	scalar controlvar`t'="school_private`t' ${controls`t'}"
	scalar othercontrols`t'="${controls`t'}"
}

*At which row do observations that switched schools start.
gen id=_n
su id if mobile
local startmobile=r(min)
local todrop `todrop' id

su ones
local totalobs=r(N)

mata
		//Import all needed scalars and variables
		phiprelim=st_matrix("phiprelim")
		y0=st_data(.,"`s'0")
		y1=st_data(.,"`s'1")
		y2=st_data(.,"`s'2")
		sid1=st_data(.,"sid1")
		sid2=st_data(.,"sid2")
		ones=st_data(.,"ones")
		controlvar0=st_strscalar("controlvar0")
		controlvar1=st_strscalar("controlvar1")
		controlvar2=st_strscalar("controlvar2")
		
		othercontrols0=st_strscalar("othercontrols0")
		othercontrols1=st_strscalar("othercontrols1")
		othercontrols2=st_strscalar("othercontrols2")
		
		countclbs = st_matrix("countclbs")
		sizeschool1=st_matrix("sizeschool1")
		sizeschool2=st_matrix("sizeschool2")
		sizeschool=sizeschool1\sizeschool2
		
		`other1'0=st_data(.,"`other1'0")
		`other2'0=st_data(.,"`other2'0")

		X0=st_data(., tokens(controlvar0))
		X1=st_data(., tokens(controlvar1))
		X2=st_data(., tokens(controlvar2))
		
		othercontrol1=st_data(., tokens(othercontrols1))
		othercontrol2=st_data(., tokens(othercontrols2))
		
		Eh=st_strscalar("Eh")
		Eh=st_data(., tokens(Eh)),ones
		
		Z0=`other1'0,`other2'0,X0,othercontrol1,othercontrol2,ones
		DX=(y1-y0),(X2-X1),ones
		DY=y2-y1
		
		//Compute weighting matrices for "optimal" estimator
		
		startsid=J(`endschool'-1,1,0)
		startsid[1,1]=1
		
		//All observations that did not switch schools have been together in both time periods
		Sigma11=2*${tausigmasqu}*J(sizeschool[1,1],sizeschool[1,1],1)
		//The weight matrix only keeps track of whether two observations should be treated as correlated when computing standard errors, the sigma matrix is where the weighting for estimation is actually going to come from.
		//Weight will have an element equal to one if the same element in Sigma is non-zero.
		Weight11=J(sizeschool[1,1],sizeschool[1,1],1)
		//Do the same thing for all schools
		for (i=`startschool'+1; i<=`endschool'; i++) {
			startsid[i-1,1]=startsid[i-2,1]+sizeschool[i-2,1]
			if (sizeschool[i-1,1]!=0) {
				Sigma11=blockdiag(Sigma11,2*${tausigmasqu}*J(sizeschool[i-1,1],sizeschool[i-1,1],1))
				Weight11=blockdiag(Weight11,J(sizeschool[i-1,1],sizeschool[i-1,1],1))
			}
		}
		//Remove the diagonal (the correct diagonal is added later)
		Sigma11=Sigma11-2*${tausigmasqu}*I(rows(Sigma11))
		Weight11=Weight11-I(rows(Sigma11))
		
		//Now deal with the observations that switched schools
		
		//Covariance with students that did not switch school (so that attended same school for at most one time period)
		Sigma12=J(`nummobile',`totalobs'-`nummobile',0)
		Weight12=J(`nummobile',`totalobs'-`nummobile',0)
		for (i=1;i<=`nummobile';i++) {
			msid1=sid1[`startmobile'+i-1,1]
			msid2=sid2[`startmobile'+i-1,1]
			if (sizeschool[msid1-1,1]!=0) {
				Sigma12[i,startsid[msid1-1,1]::startsid[msid1-1,1]+sizeschool[msid1-1,1]-1]=${tausigmasqu}*J(1,sizeschool[msid1-1,1],1)
				Weight12[i,startsid[msid1-1,1]::startsid[msid1-1,1]+sizeschool[msid1-1,1]-1]=J(1,sizeschool[msid1-1,1],1)
			}
			if (sizeschool[msid2-1,1]!=0) {	
				Sigma12[i,startsid[msid2-1,1]::startsid[msid2-1,1]+sizeschool[msid2-1,1]-1]=${tausigmasqu}*J(1,sizeschool[msid2-1,1],1)
				Weight12[i,startsid[msid2-1,1]::startsid[msid2-1,1]+sizeschool[msid2-1,1]-1]=J(1,sizeschool[msid2-1,1],1)
			}
		}
		
		//Covariance with students that also switched school (so that might have attended same school zero, one or two time periods)
		Sigma22=J(`nummobile',`nummobile',0)
		Weight22=J(`nummobile',`nummobile',0)
		for (i=1;i<=`nummobile';i++) {
			msid1=sid1[`startmobile'+i-1,1]
			msid2=sid2[`startmobile'+i-1,1]
			for (j=2;j<i;j++) {
				if (sid1[`startmobile'+j-1,1]==msid1) {
					Sigma22[j,i]=${tausigmasqu}
					Weight22[j,i]=1
				}
				if (sid1[`startmobile'+j-1,1]==msid1) {
					Sigma22[j,i]=Sigma22[j,i]+${tausigmasqu}
					Weight22[j,i]=1
				}
			}
		}
		Sigma22=Sigma22+Sigma22'
		Weight22=Weight22+Weight22'
		
		//Put all of the parts of the covariance matrix together, including the diagonal
		
		Sigma=((Sigma11\Sigma12),(Sigma12'\Sigma22))+${sigmasqm}*I(`totalobs')
		Weight=((Weight11\Weight12),(Weight12'\Weight22))+I(`totalobs')
		
		//Compute all estimators and standard errors
		
		//First stage GMM estimator in order to get optimal GMM weighting matrix to compute the Arellano and Bond estimator
		
		phiABprelim=luinv((DX')*Z0*luinv((Z0')*Z0)*((DX')*Z0)')*((DX')*Z0*luinv((Z0')*Z0)*((DY')*Z0)')
		m2ABprelim=(DY-(DX)*phiABprelim)
		
		ABweight=luinv((Z0')*diag(m2ABprelim)*diag(m2ABprelim)*Z0)
		phiAB=luinv((DX')*Z0*ABweight*((DX')*Z0)')*((DX')*Z0*ABweight*((DY')*Z0)')
		// phiAB is exactly the same as gmm (D.english- {rho}*LD.english-{alpha}*D.school_private -{beta1}*D.child_wealth -{beta2}*D.child_motherhome -{beta3}*D.child_fatherhome-{gamma}), xtinstruments(1: urdu math school_private, lags(2/.)) xtinstruments(1: child_wealth child_motherhome child_fatherhome, lags(0/.)) deriv(1/rho = -1*LD.english) deriv(1/alpha=-1*D.school_private) deriv(1/beta1=-1*D.child_wealth) deriv(1/beta2=-1*D.child_motherhome) deriv(1/beta3=-1*D.child_fatherhome)  deriv(1/gamma=-1)  wmatrix(robust)   twostep vce(cluster sid0) winitial(xt D) 
		
		//2SLS, ie not using cross-sectional dependence for optimal dependence but using average of covariates by intial school membership is also provided for reference. Usually seems to yield efficiency gains but lower than when using both stronger predictor of covariates and weighting that makes use of cross-sectional dependence.
		phi2sls=luinv((Eh')*DX)*((Eh')*DY)
		
		//"Optimal" estimator, ie which makes use of cross-sectional dependence within schools both in the weighting and in the predicting of covariates
		phiopt=luinv((Eh')*luinv(Sigma)*DX)*((Eh')*luinv(Sigma)*DY)
		
		m2opt=(DY-(DX)*phiopt)
		m22sls=(DY-(DX)*phi2sls)
		m2AB=(DY-(DX)*phiAB)
		
		//Computing estimated variance covariance matrix of the estimator
		VAB=luinv((DX')*Z0*ABweight*((DX')*Z0)')*((DX')*Z0*ABweight*(Z0')*diag(m2AB)*Weight*((DX')*Z0*ABweight*(Z0')*diag(m2AB))')*luinv((DX')*Z0*ABweight*((DX')*Z0)')'
		V2sls=luinv((Eh')*DX)*((Eh')*(diag(m22sls))*Weight*((Eh')*(diag(m22sls)))')*(luinv((Eh')*DX))'
		Vopt=luinv((Eh')*luinv(Sigma)*DX)*((Eh')*luinv(Sigma)*(diag(m2opt))*Weight*((Eh')*luinv(Sigma)*(diag(m2opt)))')*(luinv((Eh')*luinv(Sigma)*DX))'
		
		//Compute convergence criterion and update
		crit=((phiopt-phiprelim)'*(phiopt-phiprelim))^0.5
		phiprelim=phiopt
		
		m2=(DY-(DX)*phiopt)
		
		st_numscalar("crit",crit)
		st_matrix("phiprelim",phiprelim)
		st_matrix("phiopt",phiopt)
		st_matrix("Vopt",Vopt)
		
		st_matrix("phi2sls",phi2sls)
		st_matrix("V2sls",V2sls)
		
		st_matrix("phiAB",phiAB)
		st_matrix("VAB",VAB)
		
		st_store(.,"m2",m2)
		
		
end

drop `todrop' 






