global scores "english urdu math"

foreach s in $scores {
	quietly {
	mata: mata clear
	set more off
	clear all
	
	* User specified directory
	global projectdir "/mnt/home/verdierv/Spatial/Application/2009-0259_data"
	global tmp "$projectdir/tmp"
	global out "$projectdir/output"

	sysdir set PERSONAL "/mnt/home/verdierv/StataAdo"

	cd "$projectdir/"

	capture log close
	log using "$projectdir/Resultsopt`s'contempstrict.txt", text replace

	* List control variables
	global controls "child_wealth child_motherhome child_fatherhome"

	* List test score (outcome) variables
	global scores "english urdu math"

	* Load data
	use "$projectdir/finalchildren_long", clear
	
	*Create unique school id
	drop sid
	egen sid = group(child_schoolid child_mauzaid)

	*Drop all missing values:
	sort childcode round
	xtset childcode round
	gen drop=0
	foreach var in $controls $scores school_private sid {
		replace drop=1 if (`var'==.|`var'[_n+1]==.|`var'[_n+2]==.)&round==1
		replace drop=1 if (`var'==.|`var'[_n+1]==.|`var'[_n-1]==.)&round==2
		replace drop=1 if (`var'==.|`var'[_n-1]==.|`var'[_n-2]==.)&round==3
	}

	drop if drop

	keep childcode grade round $controls $scores school_private sid

	* define other subjects as possible instruments
	if "`s'"=="english" {
		local other1="math"
		local other2="urdu"
	}
	else if "`s'"=="urdu" {
		local other1="math"
		local other2="english"
	}
	else {
		local other1="english"
		local other2="urdu"
	}
	
	*Write pieces of text needed for the GMM estimation
	global ABcontrols
	global derivcontrols
	global ABcontrolslist
	local count=1
	foreach var in $controls {
		global ABcontrols $ABcontrols -{beta`count'}*D.`var'
		global derivcontrols $derivcontrols deriv(1/beta`count'=-1*D.`var')
		global ABcontrolslist $ABcontrolslist D.`var'
		local count=`count'+1
	}
	
	global L2controls
	global L2controlsbar
	global Lcontrols
	global Lcontrolsbar
	global controlsbar
	foreach var in $controls {
		global L2controls $L2controls L2.`var'
		global L2controlsbar $L2controlsbar L2.`var'bar
		global Lcontrols $Lcontrols L.`var'
		global Lcontrolsbar $Lcontrolsbar L.`var'bar
		global controlsbar $controlsbar `var'bar
	}
	
	*Create average of each control by school membership at first period
	gen sid0=sid if round==1
	replace sid0=L.sid if round==2
	replace sid0=L2.sid if round==3
	foreach var in `other1' `other2' school_private ${controls} {
		egen `var'bar=mean(`var'), by(sid0 round)
	} 
	}
	
	*Predict covariates based on instruments and averages of instruments by school membership at first period
	global listEh
	foreach var in L.`s' school_private $controls {
		reg D.`var' L2.`other1' L2.`other1'bar L2.`other2' L2.`other2'bar L2.school_private L2.school_privatebar ${L2controls} ${L2controlsbar} $Lcontrols $Lcontrolsbar $controlsbar $controls
		if "`var'"=="L.`s'" {
			predict D`s'h, xb
			global listEh $listEh D`s'h
		} 
		else {
			predict D`var'h, xb
			global listEh $listEh D`var'h
		}
	}
	quietly {
	scalar Eh="${listEh}"
	
	*Preliminar estimator needed to estimate nuisance parameters
	gmm (D.`s' - {rho}*LD.`s'-{alpha}*D.school_private ${ABcontrols}-{gamma}),instruments(1:${listEh}) deriv(1/rho = -1*LD.`s') deriv(1/alpha=-1*D.school_private) ${derivcontrols} deriv(1/gamma=-1)  wmatrix(robust) vce(robust) variables(L.`s' school_private ${controls})   twostep
	predict m
	matrix phiprelim=e(b)'
	
	*Prepare data for computing the estimators
	local tokeep childcode
	foreach var in  $controls $scores school_private sid m {
		capture gen `var'0=L2.`var'
		gen `var'1=L.`var'
		gen `var'2=`var'
		local tokeep `tokeep' `var'0 `var'1 `var'2
	}
	local tokeep `tokeep' $listEh
	
	keep if round==3
	keep `tokeep'
	}
	
	local criter=999
	forvalues iter=1/200 {
		if `criter'>0.00001 {
			qui do compute_estimates.do `s' `other1' `other2'
			di `iter'
			di phiopt[1,1]
			local criter=crit
		}
	}
	matrix list phiopt
	matrix list Vopt
	matrix list phiAB
	matrix list VAB
	matrix list phi2sls
	matrix list V2sls
}

